// Copyright (C) 2013 S&P Capital IQ
// Copyright (C) 2013 Stephen Compall
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package com.spcapitaliq.platformexecuting

import java.io.File

import scalaz.\/
import scalaz.syntax.functor._
import scalaz.syntax.std.option._

import TempStrategy.Bracket._

/** Of course, we couldn't give you a way to execute things in a
  * "platform-executing" library.  That would be too obvious.
  *
  * Our job ends when we hand you a suitable executable to use, or a
  * complaint about why we can't.  There are lots of ways to start
  * subprocesses, and they ought to be the subject of another library.
  *
  * See `TestInvoke` for one example of how you might use this.
  */
object Invoke {
  def tempInvokableExe(exe: String, resourceRoot: String,
                       env: Option[String]
                     )(implicit str: TempFileStrategy
                     ): String \/ (Option[str.Fid], File) = {
    val local = Capture.localExe(exe, env) strengthL None
    (local orElse
     (Capture.capturedExe(exe, resourceRoot)
      cata (Jar.extractTemp(_)(str tapAcquiredFile
                               ((_, tfi) => tfi.setExecutable(true))),
            local)))
  }

  /** Call `f` with a suitable `exe` location, which is only valid
    * while `f` is executing, or fail.
    */
  def withInvokableExe[A](exe: String, resourceRoot: String,
                          env: Option[String]
                        )(f: File => A
                        )(implicit str: TempFileStrategy): String \/ A =
    str bracketU (tempInvokableExe(exe = exe, resourceRoot = resourceRoot,
                                   env = env), f)
}
